import json

import requests

from .client import Client
from .error import DownloadObjectError, UploadObjectError, CombineObjectsError

class SyncClient(Client):
    def __init__(self, **kwds):
        super().__init__(**kwds)


    def download_object(self, *,
            path,
            thumbnail=False,
            thumbnail_size=None,
            thumbnail_method=None,
            thumbnail_bgcolor=None,
            image_transform=None):
        download_url = self.build_download_object_url(
                path=path,
                thumbnail=thumbnail,
                thumbnail_size=thumbnail_size,
                thumbnail_method=thumbnail_method,
                thumbnail_bgcolor=thumbnail_bgcolor,
                image_transform=image_transform)
        r = requests.get(download_url)
        response = SyncResponse(
                status_code=r.status_code,
                content_type=r.headers['content-type'],
                data=r.content)
        if response.status_code == 200:
            return response
        else:
            raise DownloadObjectError(response)


    def upload_object(self, *,
            path,
            data,
            content_type,
            image_transform=None):
        upload_url = self.build_upload_object_url(
                path=path,
                image_transform=image_transform)
        r = requests.post(
                upload_url, 
                data=data,
                headers={'Content-Type': content_type})
        response = SyncResponse(
                status_code=r.status_code,
                content_type=r.headers['content-type'],
                data=r.content)
        if response.status_code == 200 and response.content_type == 'application/json':
            return json.loads(response.data.decode())
        else:
            raise UploadObjectError(response)


    async def combine_objects(self, *,
            path,
            objects,
            content_type):
        combine_url = self.build_combine_objects_url(
                path=path)
        r = requests.post(
                combine_url, 
                json={
                    'Files': objects,
                    'Content-Type': content_type
                    })
        response = SyncResponse(
                status_code=r.status_code,
                content_type=r.headers['content-type'],
                data=r.content)
        if response.status_code == 200 and response.content_type == 'application/json':
            return json.loads(response.data.decode())
        else:
            raise CombineObjectsError(response)


class SyncResponse:
    def __init__(self, *, status_code, content_type, data):
        self.status_code=status_code
        self.content_type=content_type
        self.data=data




import datetime
import hashlib
import urllib.parse
import uuid


VERSION='1'

class Client:
    def __init__(self, *,
            warehouse,
            signer,
            secret):
        self.warehouse=warehouse
        self.signer=signer
        self.secret=secret


    def _build_url(self, *,
            path,
            action,
            thumbnail=False,
            thumbnail_size=None,
            thumbnail_method=None,
            thumbnail_bgcolor=None,
            image_transform=None,
            copy_source=None):
        url = f'https://{self.warehouse}.crushfs.net/'
        if path.startswith('/'):
            path = path[1:]
        url += f'{path}?x-cfs-version={VERSION}'
        url += f'&x-cfs-action={action}'
        if thumbnail:
            url += f'&x-cfs-thumbnail=1'
            if thumbnail_size:
                url += f'&x-cfs-thumbnail-size={thumbnail_size}'
            if thumbnail_method:
                url += f'&x-cfs-thumbnail-method={thumbnail_method}'
            if thumbnail_bgcolor:
                url += f'&x-cfs-thumbnail-bgcolor={thumbnail_bgcolor}'
        if image_transform:
            url += f'&x-cfs-image-transform={image_transform}'
        if copy_source:
            url += f'&x-cfs-copy-source={copy_source}'
        return url


    def _sign_url(self, *,
            method,
            url,
            timestamp=None,
            expiry=None,
            request_id=None,
            request_limit=None):
        url += f'&x-cfs-signer={self.signer}'
        if timestamp is None:
            timestamp=datetime.datetime.utcnow()
        url += f'&x-cfs-timestamp={timestamp.strftime("%Y%m%d%H%M%S")}'
        if expiry is None:
            expiry=60
        url += f'&x-cfs-expiry={expiry}'
        if request_id is None:
            request_id=uuid.uuid4().hex
        url += f'&x-cfs-request-id={request_id}'
        if request_limit is None:
            request_limit=1
        url += f'&x-cfs-request-limit={request_limit}'

        parsed_url = urllib.parse.urlparse(url)
        query = urllib.parse.parse_qs(parsed_url.query)
        signed_parameters = {}
        for param_name in query:
            if param_name.startswith('x-cfs-'):
                signed_parameters[param_name] = query[param_name]

        string_to_sign_elements = [method, parsed_url.netloc, parsed_url.path]
        for param_name in sorted(signed_parameters):
            string_to_sign_elements.append(signed_parameters[param_name][0])
        string_to_sign = '\n'.join(string_to_sign_elements)
        signature = hashlib.blake2b(
                string_to_sign.encode('utf-8'),
                key=self.secret.encode('utf-8')).hexdigest()

        url += f'&x-cfs-signature={signature}'

        return url


    def build_download_object_url(self, *,
            path,
            thumbnail=False,
            thumbnail_size=None,
            thumbnail_method=None,
            thumbnail_bgcolor=None,
            image_transform=None,
            timestamp=None,
            expiry=None,
            request_id=None,
            request_limit=None):
        url = self._build_url(
                path=path,
                action='download',
                thumbnail=thumbnail,
                thumbnail_size=thumbnail_size,
                thumbnail_method=thumbnail_method,
                thumbnail_bgcolor=thumbnail_bgcolor,
                image_transform=image_transform)
        signed_url = self._sign_url(
                method='GET',
                url=url,
                timestamp=timestamp,
                expiry=expiry,
                request_id=request_id,
                request_limit=request_limit)
        return signed_url


    def build_upload_object_url(self, *,
            path,
            image_transform=None,
            timestamp=None,
            expiry=None,
            request_id=None,
            request_limit=None):
        url = self._build_url(
                path=path,
                action='upload',
                image_transform=image_transform)
        signed_url = self._sign_url(
                method='POST',
                url=url,
                timestamp=timestamp,
                expiry=expiry,
                request_id=request_id,
                request_limit=request_limit)
        return signed_url
       

    def build_combine_objects_url(self, *,
            path,
            timestamp=None,
            expiry=None,
            request_id=None,
            request_limit=None):
        url = self._build_url(
                path=path,
                action='combine-files')
        signed_url = self._sign_url(
                method='POST',
                url=url,
                timestamp=timestamp,
                expiry=expiry,
                request_id=request_id,
                request_limit=request_limit)
        return signed_url




from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='crushfs',
    version='0.3.3',
    packages=find_packages(),
    install_requires=['aiohttp','requests'],
    author='David Poirier',
    author_email='david@pxcrush.net',
    description='Python 3 client for CrushFS (sync and async)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='crushfs',
    url='https://github.com/crushfs/crushfs-python-client',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
)



class DownloadObjectError(Exception):
    def __init__(self, response):
        self.message = f'{response.status_code}: {response.data.decode()}'


class UploadObjectError(Exception):
    def __init__(self, response):
        self.message = f'{response.status_code}: {response.data.decode()}'


class CombineObjectsError(Exception):
    def __init__(self, response):
        self.message = f'{response.status_code}: {response.data.decode()}'




import json

import aiohttp

from .client import Client
from .error import DownloadObjectError, UploadObjectError, CombineObjectsError

class AsyncClient(Client):
    def __init__(self, **kwds):
        super().__init__(**kwds)

    async def download_object(self, *,
            path,
            thumbnail=False,
            thumbnail_size=None,
            thumbnail_method=None,
            thumbnail_bgcolor=None,
            image_transform=None):
        download_url = self.build_download_object_url(
                path=path,
                thumbnail=thumbnail,
                thumbnail_size=thumbnail_size,
                thumbnail_method=thumbnail_method,
                thumbnail_bgcolor=thumbnail_bgcolor,
                image_transform=image_transform)
        async with aiohttp.request('GET', download_url) as r:
            response = AsyncResponse(
                    status_code=r.status,
                    content_type=r.content_type,
                    data=await r.read())
            if response.status_code == 200:
                return response
            else:
                raise DownloadObjectError(response)
    

    async def upload_object(self, *,
            path,
            data,
            content_type,
            image_transform=None):
        upload_url = self.build_upload_object_url(
                path=path,
                image_transform=image_transform)
        async with aiohttp.request(
                'POST', 
                upload_url, 
                data=data,
                headers={'Content-Type': content_type}) as r:
            response = AsyncResponse(
                    status_code=r.status,
                    content_type=r.content_type,
                    data=await r.read())
            if response.status_code == 200 and response.content_type == 'application/json':
                return json.loads(response.data.decode())
            else:
                raise UploadObjectError(response)

        
    async def combine_objects(self, *,
            path,
            objects,
            content_type):
        combine_url = self.build_combine_objects_url(
                path=path)
        async with aiohttp.request(
                'POST', 
                combine_url, 
                json={
                    'Files': objects,
                    'Content-Type': content_type
                    }) as r:
            response = AsyncResponse(
                    status_code=r.status,
                    content_type=r.content_type,
                    data=await r.read())
            if response.status_code == 200 and response.content_type == 'application/json':
                return json.loads(response.data.decode())
            else:
                raise CombineObjectsError(response)


class AsyncResponse:
    def __init__(self, *, status_code, content_type, data):
        self.content_type=content_type
        self.status_code=status_code
        self.data=data

